import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { ActorProps } from './actor.value-object';
import { Point, PointProps } from './point.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface CarpoolPathItemProps extends PointProps {
  actors: ActorProps[];
}

export class CarpoolPathItem extends ValueObject<CarpoolPathItemProps> {
  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  get actors(): ActorProps[] {
    return this.props.actors;
  }

  protected validate(props: CarpoolPathItemProps): void {
    // validate point props
    new Point({
      lon: props.lon,
      lat: props.lat,
    });
    if (props.actors.length <= 0)
      throw new ArgumentOutOfRangeException('at least one actor is required');
  }
}
