import { Injectable } from '@nestjs/common';
import { RabbitSubscribe } from '@mobicoop/message-broker-module';
import { CommandBus } from '@nestjs/cqrs';
import { CreateAdCommand } from '@modules/ad/core/application/commands/create-ad/create-ad.command';
import { Ad } from './ad.types';
import {
  AD_CREATED_MESSAGE_HANDLER,
  AD_CREATED_ROUTING_KEY,
} from '@src/app.constants';

@Injectable()
export class AdCreatedMessageHandler {
  constructor(private readonly commandBus: CommandBus) {}

  @RabbitSubscribe({
    name: AD_CREATED_MESSAGE_HANDLER,
    routingKey: AD_CREATED_ROUTING_KEY,
  })
  public async adCreated(message: string) {
    try {
      const createdAd: Ad = JSON.parse(message);
      await this.commandBus.execute(
        new CreateAdCommand({
          id: createdAd.aggregateId,
          driver: createdAd.driver,
          passenger: createdAd.passenger,
          frequency: createdAd.frequency,
          fromDate: createdAd.fromDate,
          toDate: createdAd.toDate,
          schedule: createdAd.schedule,
          seatsProposed: createdAd.seatsProposed,
          seatsRequested: createdAd.seatsRequested,
          strict: createdAd.strict,
          waypoints: createdAd.waypoints,
          pause: createdAd.pause,
        }),
      );
    } catch (error: any) {
      // do not throw error to acknowledge incoming message
      // error handling should be done in the command handler, if relevant
    }
  }
}
