import { Inject, Injectable } from '@nestjs/common';
import { MatchEntity } from './core/domain/match.entity';
import { MatchResponseDto } from './interface/dtos/match.response.dto';
import { ResponseBase } from '@mobicoop/ddd-library';
import { Journey } from './core/domain/value-objects/journey.value-object';
import { JourneyItem } from './core/domain/value-objects/journey-item.value-object';
import { ActorTime } from './core/domain/value-objects/actor-time.value-object';
import { OUTPUT_DATETIME_TRANSFORMER } from './ad.di-tokens';
import { DateTimeTransformerPort } from './core/application/ports/datetime-transformer.port';

@Injectable()
export class MatchMapper {
  constructor(
    @Inject(OUTPUT_DATETIME_TRANSFORMER)
    private readonly outputDatetimeTransformer: DateTimeTransformerPort,
  ) {}

  toResponse = (match: MatchEntity): MatchResponseDto => {
    return {
      ...new ResponseBase(match),
      adId: match.getProps().adId,
      role: match.getProps().role,
      frequency: match.getProps().frequency,
      distance: match.getProps().distance,
      duration: match.getProps().duration,
      initialDistance: match.getProps().initialDistance,
      initialDuration: match.getProps().initialDuration,
      distanceDetour: match.getProps().distanceDetour,
      durationDetour: match.getProps().durationDetour,
      distanceDetourPercentage: match.getProps().distanceDetourPercentage,
      durationDetourPercentage: match.getProps().durationDetourPercentage,
      journeys: match.getProps().journeys.map((journey: Journey) => ({
        day: new Date(
          this.outputDatetimeTransformer.fromDate(
            {
              date: journey.firstDate.toISOString().split('T')[0],
              time: journey.firstDriverDepartureTime(),
              coordinates: journey.driverOrigin(),
            },
            match.getProps().frequency,
          ),
        ).getDay(),
        firstDate: this.outputDatetimeTransformer.fromDate(
          {
            date: journey.firstDate.toISOString().split('T')[0],
            time: journey.firstDriverDepartureTime(),
            coordinates: journey.driverOrigin(),
          },
          match.getProps().frequency,
        ),
        lastDate: this.outputDatetimeTransformer.fromDate(
          {
            date: journey.lastDate.toISOString().split('T')[0],
            time: journey.firstDriverDepartureTime(),
            coordinates: journey.driverOrigin(),
          },
          match.getProps().frequency,
        ),
        steps: journey.journeyItems.map((journeyItem: JourneyItem) => ({
          duration: journeyItem.duration,
          distance: journeyItem.distance as number,
          lon: journeyItem.lon,
          lat: journeyItem.lat,
          time: this.outputDatetimeTransformer.time(
            {
              date: journey.firstDate.toISOString().split('T')[0],
              time: journeyItem.driverTime(),
              coordinates: journey.driverOrigin(),
            },
            match.getProps().frequency,
          ),
          actors: journeyItem.actorTimes.map((actorTime: ActorTime) => ({
            role: actorTime.role,
            target: actorTime.target,
          })),
        })),
      })),
    };
  };
}
