import {
  AD_DIRECTION_ENCODER,
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
} from '@modules/ad/ad.di-tokens';
import { AdMapper } from '@modules/ad/ad.mapper';
import { AdRepository } from '@modules/ad/infrastructure/ad.repository';
import { PrismaService } from '@modules/ad/infrastructure/prisma.service';
import { PostgresDirectionEncoder } from '@modules/geography/infrastructure/postgres-direction-encoder';
import { ConfigModule } from '@nestjs/config';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { Test } from '@nestjs/testing';

export async function integrationTestingModule(): Promise<{
  prismaService: PrismaService;
  adRepository: AdRepository;
}> {
  const mockMessagePublisher = {
    publish: jest.fn().mockImplementation(),
  };

  const mockLogger = {
    log: jest.fn(),
    warn: jest.fn(),
    error: jest.fn(),
  };

  const module = await Test.createTestingModule({
    imports: [
      EventEmitterModule.forRoot(),
      ConfigModule.forRoot({ isGlobal: true }),
    ],
    providers: [
      PrismaService,
      AdMapper,
      {
        provide: AD_REPOSITORY,
        useClass: AdRepository,
      },
      {
        provide: AD_MESSAGE_PUBLISHER,
        useValue: mockMessagePublisher,
      },
      {
        provide: AD_DIRECTION_ENCODER,
        useClass: PostgresDirectionEncoder,
      },
    ],
  })
    .setLogger(mockLogger)
    .compile();

  return {
    prismaService: module.get<PrismaService>(PrismaService),
    adRepository: module.get<AdRepository>(AD_REPOSITORY),
  };
}
