import { ArgumentInvalidException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';

describe('Actor time value object', () => {
  it('should create an actor time value object', () => {
    const actorTimeVO = new ActorTime({
      role: Role.DRIVER,
      target: Target.START,
      firstDatetime: new Date('2023-09-01T07:00Z'),
      firstMinDatetime: new Date('2023-09-01T06:45Z'),
      firstMaxDatetime: new Date('2023-09-01T07:15Z'),
      lastDatetime: new Date('2024-08-30T07:00Z'),
      lastMinDatetime: new Date('2024-08-30T06:45Z'),
      lastMaxDatetime: new Date('2024-08-30T07:15Z'),
    });
    expect(actorTimeVO.role).toBe(Role.DRIVER);
    expect(actorTimeVO.target).toBe(Target.START);
    expect(actorTimeVO.firstDatetime.getUTCHours()).toBe(7);
    expect(actorTimeVO.firstMinDatetime.getUTCMinutes()).toBe(45);
    expect(actorTimeVO.firstMaxDatetime.getUTCMinutes()).toBe(15);
    expect(actorTimeVO.lastDatetime.getUTCHours()).toBe(7);
    expect(actorTimeVO.lastMinDatetime.getUTCMinutes()).toBe(45);
    expect(actorTimeVO.lastMaxDatetime.getUTCMinutes()).toBe(15);
  });
  it('should throw an error if dates are inconsistent', () => {
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01T07:00Z'),
          firstMinDatetime: new Date('2023-09-01T07:05Z'),
          firstMaxDatetime: new Date('2023-09-01T07:15Z'),
          lastDatetime: new Date('2024-08-30T07:00Z'),
          lastMinDatetime: new Date('2024-08-30T06:45Z'),
          lastMaxDatetime: new Date('2024-08-30T07:15Z'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01T07:00Z'),
          firstMinDatetime: new Date('2023-09-01T06:45Z'),
          firstMaxDatetime: new Date('2023-09-01T06:55Z'),
          lastDatetime: new Date('2024-08-30T07:00Z'),
          lastMinDatetime: new Date('2024-08-30T06:45Z'),
          lastMaxDatetime: new Date('2024-08-30T07:15Z'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01T07:00Z'),
          firstMinDatetime: new Date('2023-09-01T06:45Z'),
          firstMaxDatetime: new Date('2023-09-01T07:15Z'),
          lastDatetime: new Date('2024-08-30T07:00Z'),
          lastMinDatetime: new Date('2024-08-30T07:05Z'),
          lastMaxDatetime: new Date('2024-08-30T07:15Z'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01T07:00Z'),
          firstMinDatetime: new Date('2023-09-01T06:45Z'),
          firstMaxDatetime: new Date('2023-09-01T07:15Z'),
          lastDatetime: new Date('2024-08-30T07:00Z'),
          lastMinDatetime: new Date('2024-08-30T06:45Z'),
          lastMaxDatetime: new Date('2024-08-30T06:35Z'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2024-08-30T07:00Z'),
          firstMinDatetime: new Date('2024-08-30T06:45Z'),
          firstMaxDatetime: new Date('2024-08-30T07:15Z'),
          lastDatetime: new Date('2023-09-01T07:00Z'),
          lastMinDatetime: new Date('2023-09-01T06:45Z'),
          lastMaxDatetime: new Date('2023-09-01T07:15Z'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01T07:00Z'),
          firstMinDatetime: new Date('2023-09-01T06:45Z'),
          firstMaxDatetime: new Date('2023-09-01T07:15Z'),
          lastDatetime: new Date('2024-08-31T07:00Z'),
          lastMinDatetime: new Date('2024-08-31T06:45Z'),
          lastMaxDatetime: new Date('2024-08-31T06:35Z'),
        }),
    ).toThrow(ArgumentInvalidException);
  });
});
