import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { MatchQuery } from '@modules/ad/core/domain/value-objects/match-query.value-object';

describe('Match Query value object', () => {
  it('should create a match query value object', () => {
    const matchQueryVO = new MatchQuery({
      driver: false,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: '2023-09-01',
      toDate: '2023-09-01',
      schedule: [
        {
          day: 5,
          time: '07:10',
          margin: 900,
        },
      ],
      seatsProposed: 3,
      seatsRequested: 1,
      strict: false,
      waypoints: [
        {
          lat: 48.689445,
          lon: 6.17651,
        },
        {
          lat: 48.21548,
          lon: 5.65874,
        },
      ],
      algorithmType: AlgorithmType.PASSENGER_ORIENTED,
      remoteness: 15000,
      useProportion: true,
      proportion: 0.3,
      useAzimuth: true,
      azimuthMargin: 10,
      maxDetourDistanceRatio: 0.3,
      maxDetourDurationRatio: 0.3,
    });
    expect(matchQueryVO.driver).toBe(false);
    expect(matchQueryVO.passenger).toBe(true);
    expect(matchQueryVO.frequency).toBe(Frequency.PUNCTUAL);
    expect(matchQueryVO.fromDate).toBe('2023-09-01');
    expect(matchQueryVO.toDate).toBe('2023-09-01');
    expect(matchQueryVO.schedule?.length).toBe(1);
    expect(matchQueryVO.seatsProposed).toBe(3);
    expect(matchQueryVO.seatsRequested).toBe(1);
    expect(matchQueryVO.strict).toBe(false);
    expect(matchQueryVO.waypoints.length).toBe(2);
    expect(matchQueryVO.algorithmType).toBe(AlgorithmType.PASSENGER_ORIENTED);
    expect(matchQueryVO.remoteness).toBe(15000);
    expect(matchQueryVO.useProportion).toBe(true);
    expect(matchQueryVO.proportion).toBe(0.3);
    expect(matchQueryVO.useAzimuth).toBe(true);
    expect(matchQueryVO.azimuthMargin).toBe(10);
    expect(matchQueryVO.maxDetourDistanceRatio).toBe(0.3);
    expect(matchQueryVO.maxDetourDurationRatio).toBe(0.3);
  });
});
