import {
  ArgumentInvalidException,
  ArgumentOutOfRangeException,
} from '@mobicoop/ddd-library';
import { Step } from '@modules/ad/core/domain/value-objects/step.value-object';

describe('Step value object', () => {
  it('should create a step value object', () => {
    const stepVO = new Step({
      lat: 48.689445,
      lon: 6.17651,
      duration: 150,
      distance: 12000,
    });
    expect(stepVO.duration).toBe(150);
    expect(stepVO.distance).toBe(12000);
    expect(stepVO.lat).toBe(48.689445);
    expect(stepVO.lon).toBe(6.17651);
  });
  it('should throw an exception if longitude is invalid', () => {
    expect(() => {
      new Step({
        lat: 48.689445,
        lon: 186.17651,
        duration: 150,
        distance: 12000,
      });
    }).toThrow(ArgumentOutOfRangeException);
    expect(() => {
      new Step({
        lat: 48.689445,
        lon: -186.17651,
        duration: 150,
        distance: 12000,
      });
    }).toThrow(ArgumentOutOfRangeException);
  });
  it('should throw an exception if latitude is invalid', () => {
    expect(() => {
      new Step({
        lat: 248.689445,
        lon: 6.17651,
        duration: 150,
        distance: 12000,
      });
    }).toThrow(ArgumentOutOfRangeException);
    expect(() => {
      new Step({
        lat: -148.689445,
        lon: 6.17651,
        duration: 150,
        distance: 12000,
      });
    }).toThrow(ArgumentOutOfRangeException);
  });
  it('should throw an exception if distance is invalid', () => {
    expect(() => {
      new Step({
        lat: 48.689445,
        lon: 6.17651,
        duration: 150,
        distance: -12000,
      });
    }).toThrow(ArgumentInvalidException);
  });
  it('should throw an exception if duration is invalid', () => {
    expect(() => {
      new Step({
        lat: 48.689445,
        lon: 6.17651,
        duration: -150,
        distance: 12000,
      });
    }).toThrow(ArgumentInvalidException);
  });
});
